package PITA::Image::Task;

# A generic task to be run in the image

use strict;

use vars qw{$VERSION};
BEGIN {
	$VERSION = '0.20';
}

sub new { die "CODE INCOMPLETE" }

sub run { die "CODE INCOMPLETE" }

sub result { die "CODE INCOMPLETE" }

1;
